require 'foodtaster/config'
require 'foodtaster/rspec'

require 'logger'

module Foodtaster
  autoload :Client,        'foodtaster/client'
  autoload :ServerProcess, 'foodtaster/server_process'
  autoload :Vm,            'foodtaster/vm'
  autoload :RSpecRun,      'foodtaster/rspec_run'

  class << self
    def logger
      @logger ||= Logger.new(STDOUT).tap do |log|
        log_level = ENV['FT_LOGLEVEL'] || self.config.log_level.to_s.upcase
        log.level = Logger.const_get(log_level)

        log.formatter = proc do |severity, datetime, progname, msg|
          "[FT #{severity}]: #{msg}\n"
        end
      end
    end

    def config
      @config ||= Config.default
    end

    def configure(&config_block)
      Foodtaster::RSpec.configure
      config.configure(&config_block) if config_block
    end
  end
end
