require 'timeout'

RSpec::Matchers.define :have_running_process do |process|
  match do |vm|
    vm.execute("pgrep -f #{process}").successful?
  end

  failure_message_for_should do |vm|
    "expected that #{vm.name} should have running process '#{process}'"
  end

  failure_message_for_should_not do |vm|
    "expected that #{vm.name} should not have running process '#{process}'"
  end

  description do
    "have running process '#{process}'"
  end
end

RSpec::Matchers.define :have_package do |package|
  match do |vm|
    vm.execute("dpkg --status #{package}").successful?
  end

  failure_message_for_should do |vm|
    "expected that #{vm.name} should have installed package '#{package}'"
  end

  failure_message_for_should_not do |vm|
    "expected that #{vm.name} should not have installed package '#{package}'"
  end

  description do
    "have installed package '#{package}'"
  end
end

RSpec::Matchers.define :listen_port do |port|
  match do |vm|
    vm.execute("lsof -i :#{port.to_s}").should be_successful
  end

  failure_message_for_should do |vm|
    "expected that #{vm.name} should listen to port '#{port}'"
  end

  failure_message_for_should_not do |vm|
    "expected that #{vm.name} should not listen to port '#{port}'"
  end

  description do
    "listen port '#{port}'"
  end
end

RSpec::Matchers.define :have_group do |group|
  match do |vm|
    vm.execute("cat /etc/group | cut -d: -f1 | grep \"\\<#{group}\\>\"").successful?
  end

  failure_message_for_should do |vm|
    "expected that #{vm.name} should have group '#{group}'"
  end

  failure_message_for_should_not do |vm|
    "expected that #{vm.name} should not have group '#{group}'"
  end

  description do
    "have group '#{group}'"
  end
end

def wait_until(_timeout = 5)
  begin
    timeout _timeout do
      until (result = yield)
        sleep 0.5
      end
      result
    end
  rescue Timeout::Error
    nil
  end
end

RSpec::Matchers.define :be_successful do |opts = {}|
  match do |command|
    if command.respond_to?(:call)
      wait_until(opts[:timeout] || 5) { command.call.successful? }
    else
      command.successful?
    end
  end
end
